#!/bin/bash

# Скрипт помогает установить пакеты SoftWLC на один сервер
# Предварительно устанавливает Java, mysql-server, tomcat, curl, libcurl и прочие необходимые пакеты из зависимостей
# Затем последовательно устанавливает все нужные пакеты комплекса:
#
# eltex-oui-list           : Содержит список соответствий MAC-адресов и производителей оборудования
# eltex-axis               : Фреймворк для работы службы SOAP/XML (Apache Axis2)
# eltex-ems-db             : Разворачивает схему в БД MySQL для EMS
# eltex-radius-db          : Разворачивает схему в БД MySQL для RADIUS-сервера
# eltex-auth-service-db    : Разворачивает схему в БД MySQL для сервиса авторизации
# eltex-ems                : Серверная и клиентская часть СУ EMS
# eltex-radius             : RADIUS-сервер
# eltex-radius-nbi         : Северный мост (northbound) для стыка SoftWLC с вышестоящими OSS/BSS
# eltex-ngw                : Сервис, предоставляющий возможность отправки уведомлений (СМС, звонков), используемый другими пакетами комплекса
# eltex-apb                : Сервис для взаимодействия точек доступа
# eltex-pcrf               : Служба управления политиками доступа (используется BRAS)
# eltex-logging-service    : Микросервис журналирования операций
# eltex-mercury            : Сервис по управлению Hotspot пользователями
# eltex-portal             : Портал для авторизации клиентов WiFi в схеме 'Hotspot'
# eltex-portal-constructor : Web-приложение для создания и редактирования порталов для авторизации
# eltex-wifi-cab           : Личный кабинет оператора услуги Wi-Fi
# eltex-sorm2-replicator * : Служба для обогащения и пересылки radius acct в региональные съемники СОРМ-2
#                          : И для формирования ACCT отчетов СОРМ-3
# eltex-wifi-sa **         : Сервис-активатор ОТТ
# eltex-ott-mac-checker ** : Сервис проверки MAC-адреса на принадлежность к OTT домену
# eltex-ott-paul **        : Сервис ОТТ, генератор паролей
# eltex-bonnie ***         : Сервис статистика аккаунтинга из ClickHouse
# eltex-bonnie-db ***      : Разворачивает схему в БД ClickHouse для сервиса статистики
# eltex-disconnect-service : Сервис обеспечивает немедленное прерывание сессии пользователя
# eltex-doors              : Обеспечивает авторизацию пользователя/сервиса внутри ядра (внутреннее взаимод.)
# eltex-johnny             : REST API к Mercury для управления Enterprise и Hotspot пользователями
# eltex-bruce              : Сервис-планировщик задач, обеспечивает запуск задач по установленному расписанию
# eltex-jobs               : Сервис-исполнитель задач, выполняет задачи, запущенные eltex-bruce
# eltex-airtune            : Сервис обеспечивает RRM (Radio Resource Management) и Client Load Balancing
#
# * - не устанавливаются по умолчанию, но могут быть установленны из репозитория.
# ** - не устанавливаются по умолчанию, но могут быть установлены с флагом WITH_OTT или отдельно из репо.
# *** - не устанавливаются по умолчанию, но могут быть установлены с флагом ClickHouse или отдельно из репо.

# Версия: SoftWLC 1.25, EMS 3.29
# Целевая ОС: bionic (ubuntu 18.04), stretch (debian 9), 'orel' (astralinuxce 2.12)
# Автор: Абаренов ВП
# ООО Предприятие Элтекс
# Новосибирск, 2022

# Модификаторы запуска скрипта:
# --clickhouse    : добавить установку БД clickhouse и включить использование её дня работы с RADIUS accounting
# --ott           : добавить установку сервисов OTT
# --test-ports    : Режим "только протестировать открытые порты Платформы SoftWLC", без установки пакетов.
#                   Может применяться совместно с модификаторами --ott, --clickhouse
# --dhcp          : установить пакет 'isc-dhcp-server' в его дефолтной конфигурации;
# --force-old-conffiles : в этом режиме при обновлении dpkg будет автоматически принимать решение
#                         использовать старые conf файлы
# --force-new-conffiles : в этом режиме dpkg при обновлении будет автоматически принимать решение
#                         использовать новые conf файлы
# serverip=10.20.30.40 : задать IPv4 адрес сервера, который будет прописан в конфигурацию для перехода ЛК-КП-ЛК
#                        можно использовать serverip или SERVERIP. Нельзя задавать несуществующие адреса,
#                        нельзя задавать доменное имя или 127.0.0.1. Если у сервера один интерфейс (один адрес),
#                        то можно не задавать, скрипт вычислит и подставит автоматически.
# emsip=11.12.13.14 : задать IPv4 адрес сервера в управляющей сети. Тот адрес по которому ТД и другое оборудование
#                     будут обращаться к СУ по протоколам TFTP, FTP, HTTP, AirTune и т.д.
#                     этот адрес скрипт пропишет в собственные настройки сервиса eltex-ems в его БД
#
# Ответы для автоматической установки
# Измените при необходимости

# Имя пользователя администратора MySQL
export ANSWER_SOFTWLC_MYSQL_USER=root
# Пароль администратора MySQL
export ANSWER_SOFTWLC_MYSQL_PASSWORD=root
# Имя пользователя администратора SoftWLC
export ANSWER_AUTH_SERVICE_ADMIN_USER=admin
# Пароль администратора SoftWLC
export ANSWER_AUTH_SERVICE_ADMIN_PASSWORD=password
# Пароль служебного пользователя SoftWLC (softwlc_service)
export ANSWER_SOFTWLC_SERVICE_USER_PASSWORD=softwlc
# Корневой домен
export ANSWER_SOFTWLC_ROOT_DOMAIN=root
# Язык EMS по умолчанию: 1 - русский, 2 - английский
export ANSWER_EMS_LANG=1
# Максимальное количество ОЗУ, выделяемое EMS (в МБ)
export ANSWER_EMS_MAX_HEAP=1024
# Максимальное количество ОЗУ, выделяемое Tomcat (в МБ)
export ANSWER_TOMCAT_MAX_HEAP=1024
# Код создаваемого тарифа
export ANSWER_RADIUS_TARIFF_CODE=default
# Генерировать ли сертификат для сервера RADIUS
export ANSWER_NBI_MAKE_SERVER_CERTIFICATE=1
# Срок действия серверного сертификата RADIUS
export ANSWER_NBI_SERVER_CERTIFICATE_PERIOD=3650
# Пароль от закрытого ключа серверного сертификата RADIUS
export ANSWER_NBI_SERVER_CERTIFICATE_KEY=1234
# Ограничения пользователя javauser при многохостовой установке ClickHouse
export ANSWER_SOFTWLC_CLICKHOUSE_MULTIHOST=N
# Генерация ключей для eltex-doors
export ANSWER_GENERATE_KEYS=N

# Установить пакеты eltex-wifi-sa и eltex-ott-mac-checker
export WITH_OTT=0

# Не рекомендуется редактировать
export ANSWER_SOFTWLC_LOCAL=1
export ANSWER_EMS_REPLACE_CONF=1
export ANSWER_EMS_ACCESS_TYPE_DOMAIN=1
export ANSWER_RADIUS_MAKE_TARIFF=1
export ANSWER_RADIUS_TARIFF_PORTAL=1
export ANSWER_SOFTWLC_SERVICE_USER_LOGIN=softwlc_service
export ANSWER_RADIUS_DB_UPDATE_CRON=1
export ANSWER_SOFTWLC_SCHEDULE_BACKUP=Y

# Настройка автоматического ответа на интерактивные вопросы
export DEBIAN_FRONTEND="noninteractive"

# Настройки MYSQL пользователя для сервисов EMS
export EMS_USER="javauser"
export EMS_PASSWORD="javapassword"

# Public Eltex production repo
ELTEX_PUBLIC_REPO="http://archive.eltex-co.ru/wireless"
# Private (internal) repo
ELTEX_PRIVATE_REPO="http://lab3-repo.eltex.loc:2088"
# Переменная, которая является рабочей. Внутри скрипта работа идёт с ней в зависимости от параметров вызова скрипта
ELTEX_REPO=${ELTEX_PUBLIC_REPO}

## Установить ClickHouse и пакеты eltex-bonnie, eltex-bonnie-db
INSTALL_CLICKHOUSE=0

# Режимы обновления dpkg
OLD_CONF_FILES=0
NEW_CONF_FILES=0

## Установить пакет isc-dhcp-server
INSTALL_DHCP=0

# Наименование программ (определяют путь в репозитории)
SOFTWLC_VERSION="softwlc-1.25"
RADIUS_VERSION="radius-1.25"

REPO_GPG_KEY_ADDR="${ELTEX_REPO}/repo.gpg.key"
SOFTWLC_DISTRIBUTION="$SOFTWLC_VERSION" # * параметр зависит от версии ubuntu (будет переопределён в коде)
SOFTWLC_REPO_SOURCES="deb [arch=amd64] $ELTEX_REPO $SOFTWLC_VERSION main"

NGINX_CONFIG_FILE="softwlc_1.25_nginx.conf"
# Режим "только тестируем порты и выходим", по умолчанию выключен
TEST_PORTS_MODE=0

red=$(tput setaf 1)
green=$(tput setaf 2)
reset=$(tput sgr 0)

# Вендор JVM
JAVA_VENDOR="openjdk"
TOMCAT_PACKET_NAME="tomcat7" # по умолчанию ставить tomcat7, в более поздних (>= bionic) ставить 8
LIBCURL_PACKET_NAME="libcurl4"

# Пути к файлам лицензий
AIRTUNE_LICENSE_PATH="/etc/eltex-airtune/licence"
AIRTUNE_LICENSE_FILES="$AIRTUNE_LICENSE_PATH/licence*.xml"
EMS_LICENSE_PATH="/usr/lib/eltex-ems/conf/licence"
EMS_LICENSE_FILES="$EMS_LICENSE_PATH/licence*.xml"

# Массив внешних IP сервера (весь ifconfig, кроме 127.0.0.1)
EXTERNAL_IPS=()
# Переменная, куда сохраняется переданное в аргументах значения для адреса сервера, подставляемое в КП/ЛК
# Если не передали в аргументах, то ждёт выбор пользователя.
external_ip=""

# Список поддерживаемых кодовых имён операционных систем
LINUX_CODENAMES="
  xenial
  bionic
  stretch
  buster
  orel"

# Список поддерживаемых дистрибуторов операционных систем
LINUX_DISTRIBUTORS="
  debian
  ubuntu
  astralinuxce"

# Прервать установку при ошибках
set -e

echo "${green}Installation started for $SOFTWLC_VERSION, from $ELTEX_REPO${reset}"

# Метод удаляет файл с Элтекс репо, т.к. при недоступности репо скрипт не
# может выполнить ни одну из функций
clean_eltex_repo() {
  # удалить старый репозиторий (ems, softwlc), если он есть
  if [[ -f "/etc/apt/sources.list.d/eltex.list" ]]; then
    rm /etc/apt/sources.list.d/eltex.list
  fi
}

check_memory() {
  ram=$(grep MemTotal /proc/meminfo | awk '{printf "%d", $2 / 1000}')

  if [[ $ram -lt 8000 ]]; then
    echo "${red}You have $ram MB of RAM. You need 8 GB of RAM and 10 GB of free hard disk space, installation aborted!${reset}"
    exit 1
  fi

  free_space=$(df / | grep / | awk '{printf "%d", $4 / 1000}')

  if [[ $free_space -lt 10000 ]]; then
    echo "${red}You have $free_space MB of free hard disk space. 10 GB required for installation. Installation aborted!${reset}"
    exit 1
  fi
}

update_repo_related_vars() {
  REPO_GPG_KEY_ADDR="${ELTEX_REPO}/repo.gpg.key"
  SOFTWLC_REPO_SOURCES="deb [arch=amd64] $ELTEX_REPO $SOFTWLC_VERSION main"
}

# проверить наличие обязательного файла (без него не будет настроен nginx, а значит не заработает половина сервисов)
check_nginx_config() {
  if [[ -f "$NGINX_CONFIG_FILE" ]]; then
    echo "File '$NGINX_CONFIG_FILE' found."
    FILESIZE=$(stat -c%s "$NGINX_CONFIG_FILE")
    if [[ $FILESIZE -gt 0 ]]; then
      echo "Size of $NGINX_CONFIG_FILE = $FILESIZE bytes."
    else
      echo "${red}File '$NGINX_CONFIG_FILE' is empty, installation aborted!${reset}"
      exit 1
    fi
  else
    echo "${red}File '$NGINX_CONFIG_FILE' not found, installation aborted!${reset}"
    exit 1
  fi
}

set_rsyslog_mysql_silent_mode() {
  # rsyslog-mysql login@password settings (default: login=root, password=root)
  echo "rsyslog-mysql   rsyslog-mysql/dbconfig-install  boolean true" | debconf-set-selections
  echo "rsyslog-mysql   rsyslog-mysql/mysql/app-pass    password $ANSWER_SOFTWLC_MYSQL_PASSWORD" | debconf-set-selections
  echo "rsyslog-mysql   rsyslog-mysql/app-password-confirm      password $ANSWER_SOFTWLC_MYSQL_PASSWORD" | debconf-set-selections
  echo "rsyslog-mysql   rsyslog-mysql/password-confirm  password $ANSWER_SOFTWLC_MYSQL_PASSWORD" | debconf-set-selections
  echo "rsyslog-mysql   rsyslog-mysql/mysql/admin-pass  password $ANSWER_SOFTWLC_MYSQL_PASSWORD" | debconf-set-selections
  echo "rsyslog-mysql   rsyslog-mysql/remote/port       string " | debconf-set-selections
}

install() {
    # https://man7.org/linux/man-pages/man1/dpkg.1.html - info about used options
    if [[ ${OLD_CONF_FILES} == 1 ]]; then
       apt-get --yes  -o Dpkg::Options::="--force-confold" install "$@"
    elif [[ ${NEW_CONF_FILES} == 1 ]]; then
       apt-get --yes  -o Dpkg::Options::="--force-confnew" install "$@"
    else
       apt-get --yes install "$@"
    fi
}

stop() {
  # add '|| true' - to ignore error
  service "$@" stop || true
}

restart() {
  service "$@" restart
}

start() {
  service "$@" start
}

reload() {
  service "$@" reload || true
}

update() {
  apt-get -y update || true
}

# Перезаписывает файловые лимиты для службы mysql
function replace_open_files_for_mysql() {
  local DIR="/etc/systemd/system/mysql.service.d"
  local FILE_OVERRIDE="$DIR/override.conf"
  if [ -f "$FILE_OVERRIDE" ]; then
    rm "$FILE_OVERRIDE"
  fi

  if [ ! -d "$DIR" ]; then
    mkdir -p "$DIR"
  fi

  echo "[Service]" >"$FILE_OVERRIDE"
  echo "LimitNOFILE=1617596" >>"$FILE_OVERRIDE"
  echo "LimitNOFILESoft=1617596" >>"$FILE_OVERRIDE"
  echo "File '$FILE_OVERRIDE' replaced with new configuration"
  systemctl daemon-reload
  restart mysql
}

# Раскомментировать модули приёма данных из сети, если они закоментированы в главном конфиге службы
function rsyslog_uncomment_network_mod() {
  # uncomment
  # sed -i '/<pattern>/s/^#//g' file
  # comment
  # sed -i '/<pattern>/s/^/#/g' file
  local FILE="/etc/rsyslog.conf"

  sed -i '/imudp/s/^#//g' $FILE
  sed -i '/imtcp/s/^#//g' $FILE
}

# Контроль открытого порта (без контроля службы)
# Вызов функции: check_port "8080" result_var
# В результат будет помещён 0 - OK (порт открыт) или 1 - Ошибка (порта нет)
function check_port() {
  local __resultvar=$2
  local myresult='0'
  if [[ $(netstat -pna | grep -s ":$1") ]]; then
    echo "${green}Checking port '$1' - passed${reset}"
    myresult='0'
  else
    echo "${red}Checking port '$1' - error${reset}"
    myresult='1'
  fi
  eval ${__resultvar}="'$myresult'"
}

# Метод проверки открытых портов. Количество попыток проверки передается как входной аргумент.
# Задержка между попытками - 5 секунд.
# Если по завершении проверки какой-либо из портов до сих пор закрыт - функция завершит выполнение с кодом 2.
# Вызов функции: check_all_ports "25" "1" "1", где первый аргумент - максимальное количество попыток проверки.
# Второй и третий аргументы - флаги для проверки портов сервисов для OTT и ClickHouse соответственно.
function check_all_ports() {
  local __CHECK_COUNT=$1
  local __WITH_OTT=$2
  local __WITH_CLICKHOUSE=$3
  declare -A service_ports_to_check
  declare -A service_ports_failed_results
  service_ports_to_check=(
    ['eltex-ems']='9310'
    ['eltex-ems-snmp-api']='162'
    ['eltex-doors']='9097'
    ['eltex-pcrf']='7070'
    ['eltex-apb']='8090'
    ['eltex-ngw']='8040'
    ['eltex-portal']='9000'
    ['eltex-portal-constructor']='9001'
    ['eltex-bruce']='8008'
    ['eltex-jobs']='9696'
    ['eltex-wifi-cab']='8083'
    ['eltex-airtune']='8089'
    ['eltex-logging-service']='9099'
    ['eltex-mercury']='6565'
    ['eltex-disconnect-service']='9096'
    ['eltex-johnny']='9100'
  )

  if [[ "${__WITH_OTT}" == "1" ]]; then
    service_ports_to_check['eltex-ott-mac-checker']='8088'
    service_ports_to_check['eltex-paul']='8098'
    service_ports_to_check['eltex-wifi-sa']='8043'
  fi

  if [[ "${__WITH_CLICKHOUSE}" == "1" ]]; then
    service_ports_to_check['eltex-bonnie']='9070'
  fi

  CHECK_PORTS_PASSED="1"
  for i in $(seq 1 ${__CHECK_COUNT}); do
    echo "Attempt ${i}/${__CHECK_COUNT}"
    for service in "${!service_ports_to_check[@]}"; do
      check_port "${service_ports_to_check[$service]}" check_port_result
      if [[ "$check_port_result" == "1" ]]; then
        service_ports_failed_results[$service]=$check_port_result
      else
        unset service_ports_to_check[$service]
        unset service_ports_failed_results[$service]
      fi
    done
    echo
    if [[ "${#service_ports_failed_results[@]}" != "0" ]]; then
      sleep 5
    else
      CHECK_PORTS_PASSED="0"
      break
    fi
  done
  if [[ "$CHECK_PORTS_PASSED" != "0" ]]; then
    for service in "${!service_ports_failed_results[@]}"; do
      echo "${red}Component ${service} out of service (${service_ports_to_check[${service}]} not opened)${reset}"
    done
    exit 2
  fi
}

# Полностью переписать конфиг для плагина rsyslog-mysql
function rsyslog_mysql_replace_config() {
  # Это оригинальный файл: /etc/rsyslog.d/mysql.conf
  # Нам нужен файл, который выполнится ранее дефолтного /etc/rsyslog.d/50-default.conf
  # обработает цепочку "сохранить данные из сети" и прервёт обработку не загружая файловые логгеры
  # Для этого нужно создать конфиг mysql с именем 10-mysql.conf, переопределить там шаблон сохранения SQL
  # и прервать цепочку

  # Удалим первоначальный файл
  local DIR="/etc/rsyslog.d"
  local FILE_OVERRIDE="$DIR/mysql.conf"
  if [ -f "$FILE_OVERRIDE" ]; then
    rm "$FILE_OVERRIDE"
  fi

  # Удалим целевой файл, если как-то его создавали ранее
  local FILE_OVERRIDE="$DIR/10-mysql.conf"
  if [ -f "$FILE_OVERRIDE" ]; then
    rm "$FILE_OVERRIDE"
  fi

  {
    echo "### Configuration file for rsyslog-mysql"
    echo "### Changes are preserved"
    echo "\$template tpl,\"insert into SystemEvents (Message, Facility,FromHost, FromHostIp, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag) values ('%msg%', %syslogfacility%, '%HOSTNAME%', INET_ATON('%fromhost-ip%'), %syslogpriority%, '%timereported:::date-mysql%', '%timegenerated:::date-mysql%', %iut%, '%syslogtag%')\",SQL"
    echo "module (load=\"ommysql\")"
    echo ":fromhost-ip, !isequal, \"127.0.0.1\" action(type=\"ommysql\" server=\"localhost\" db=\"Syslog\" uid=\"rsyslog\" pwd=\"root\" Template=\"tpl\")"
    echo "& stop"
  } >>"$FILE_OVERRIDE"
}

# Функция проверяет установлена ли расширенная схема Syslog в базе данных (с партициями и новыми полями)
# Возвращает "0", если схема соответствует эталону;
# Возвращает "1", если схема не соoтветствует эталону;
function rsyslog_check_extended_database() {
  # Запомнить переменную "на входе", чтобы передать в неё значение "на выходе"
  local __resultvar=$1
  local myresult="0"

  local DB="Syslog"
  local TABLE="SystemEvents"
  local PART29="PARTITION p29"
  local myvar=""

  # Проверка наличия схемы
  myvar=$(mysql --login-path=local -DSyslog -u$ANSWER_SOFTWLC_MYSQL_USER -p$ANSWER_SOFTWLC_MYSQL_PASSWORD -se "SHOW DATABASES LIKE '$DB';")
  # echo "MySQL answer = "${myvar}
  if [[ ! $myvar == *${DB}* ]]; then
    echo "${red}Database '$DB' does not exists${reset}"
    myresult="1"
  else
    myresult="0"
  fi

  # Проверка наличия таблицы
  if [[ ! "$myresult" == "1" ]]; then
    myvar=$(mysql --login-path=local -D$DB -u$ANSWER_SOFTWLC_MYSQL_USER -p$ANSWER_SOFTWLC_MYSQL_PASSWORD -se "SHOW TABLES LIKE '$TABLE';")
    # echo "MySQL answer = "${myvar}
    if [[ ! $myvar == *${TABLE}* ]]; then
      echo "${red}Table '$TABLE' does not exists${reset}"
      myresult="1"
    else
      myresult="0"
    fi
  fi

  # Проверка наличия партиционирования в схеме - единственный положительный результат
  if [[ ! "$myresult" == "1" ]]; then
    myvar=$(mysql --login-path=local -D$DB -u$ANSWER_SOFTWLC_MYSQL_USER -p$ANSWER_SOFTWLC_MYSQL_PASSWORD -se "SHOW CREATE TABLE $TABLE;")
    # echo "MySQL answer = "${myvar}
    if [[ $myvar == *${PART29}* ]]; then
      myresult="0"
    else
      myresult="1"
    fi
  fi
  # Возвращаем результат
  if [[ "$__resultvar" ]]; then
    eval $__resultvar="'$myresult'"
  else
    echo "$myresult"
  fi
}

# Создаем расширенную схему Syslog, которую использует rsyslog-mysql и показывает сервер и GUI eltex-ems
function rsyslog_create_extended_database() {

  mysql -u$ANSWER_SOFTWLC_MYSQL_USER -p$ANSWER_SOFTWLC_MYSQL_PASSWORD <<MY_QUERY
-- MySQL dump 10.13  Distrib 5.1.67, for debian-linux-gnu (i486)
--
-- Host: localhost    Database: Syslog
-- ------------------------------------------------------
-- Server version       5.1.67-0ubuntu0.10.04.1

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;

--
-- Current Database: Syslog
--

drop database if exists Syslog;

CREATE DATABASE /*!32312 IF NOT EXISTS*/ Syslog DEFAULT CHARACTER SET utf8;

USE Syslog;

-- DROP TABLE IF EXISTS SystemEvents;
/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
-- DROP TABLE IF EXISTS SystemEvents;
/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE SystemEvents (
  ID bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  CustomerID bigint(20) DEFAULT NULL,
  ReceivedAt datetime NOT NULL DEFAULT '1971-01-01 00:00:01',
  DeviceReportedTime datetime DEFAULT NULL,
  Facility smallint(6) DEFAULT NULL,
  Priority smallint(6) DEFAULT NULL,
  FromHost varchar(60) DEFAULT NULL,
  Message text,
  InfoUnitID int(11) DEFAULT NULL,
  SysLogTag varchar(60) DEFAULT NULL,
  FromHostIp INT UNSIGNED,
  PRIMARY KEY (ID,ReceivedAt,FromHostIp)
) ENGINE=MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8
/*!50100 PARTITION BY RANGE ( DAYOFMONTH(ReceivedAt))
SUBPARTITION BY HASH(FromHostIp) SUBPARTITIONS 33
(PARTITION p1 VALUES LESS THAN (2),
 PARTITION p2 VALUES LESS THAN (3),
 PARTITION p3 VALUES LESS THAN (4),
 PARTITION p4 VALUES LESS THAN (5),
 PARTITION p5 VALUES LESS THAN (6),
 PARTITION p6 VALUES LESS THAN (7),
 PARTITION p7 VALUES LESS THAN (8),
 PARTITION p8 VALUES LESS THAN (9),
 PARTITION p9 VALUES LESS THAN (10),
 PARTITION p10 VALUES LESS THAN (11),
 PARTITION p11 VALUES LESS THAN (12),
 PARTITION p12 VALUES LESS THAN (13),
 PARTITION p13 VALUES LESS THAN (14),
 PARTITION p14 VALUES LESS THAN (15),
 PARTITION p15 VALUES LESS THAN (16),
 PARTITION p16 VALUES LESS THAN (17),
 PARTITION p17 VALUES LESS THAN (18),
 PARTITION p18 VALUES LESS THAN (19),
 PARTITION p19 VALUES LESS THAN (20),
 PARTITION p20 VALUES LESS THAN (21),
 PARTITION p21 VALUES LESS THAN (22),
 PARTITION p22 VALUES LESS THAN (23),
 PARTITION p23 VALUES LESS THAN (24),
 PARTITION p24 VALUES LESS THAN (25),
 PARTITION p25 VALUES LESS THAN (26),
 PARTITION p26 VALUES LESS THAN (27),
 PARTITION p27 VALUES LESS THAN (28),
 PARTITION p28 VALUES LESS THAN (29),
 PARTITION p29 VALUES LESS THAN (30),
 PARTITION p30 VALUES LESS THAN (31),
 PARTITION p31 VALUES LESS THAN MAXVALUE) */;
/*!40101 SET character_set_client = @saved_cs_client */;


-- DROP TABLE IF EXISTS SystemEventsProperties;
/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE SystemEventsProperties (
  ID bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  SystemEventID bigint(20) DEFAULT NULL,
  ParamName varchar(255) DEFAULT NULL,
  ParamValue text,
  PRIMARY KEY (ID)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
/*!40101 SET character_set_client = @saved_cs_client */;
/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;
MY_QUERY
}

# Контроль открытого порта для определённой службы в режиме TCP (+ контроль LISTEN)
# Пример вызова функции: check_port_service_tcp "8080" "nginx" "nginx" result_var
# Пример вызова функции: check_port_service_tcp "8080" "java" "tomcat" result_var
# $1 - номер порта (строкой);
# $2 - название сервиса (java - для всех java приложений) или 'docker-proxy' для схемы с контейнерами;
# $3 - описание сервиса, например, 'tomcat' или 'Captive Portal' для Java приложения;
# $4 - В результат будет помещена строка: "0" - это OK (порт открыт) или "1" - это ошибка (порта нет)
function check_port_service_tcp() {
  local __resultvar="$4"
  local myresult='0'
  local port=":$1"
  local service="$2"
  local service_descr="$3"
  local variable=$(netstat -tlpna | awk '{if (($1 ~ /tcp/ || $1 ~ /tcp6/) && $4 ~ "'"$port"'" && $6 ~ /LISTEN/ && ($7 ~ "'"$service"'" || $7 ~ /docker-proxy/)) print "passed"}')
  if [[ $variable =~ "passed" ]]; then
    echo "${green}Checking tcp port '$1' for application '$2' ($service_descr) - passed${reset}"
    myresult='0'
  else
    echo "${red}Checking tcp port '$1' for application '$2' ($service_descr) - error${reset}"
    myresult='1'
  fi
  eval ${__resultvar}="'$myresult'"
}

# Выполняет GRANT на заданной БД
# $1 - БД
# $2 - привелегия (ALL, FILE, etc.)
# $3 - пользователь
# $4 - пароль
# $5 - таблица
grant() {
  if [[ -z $5 ]]; then
    grant_advanced "$1" "$2" "$3" "$4" "*"
  else
    grant_advanced $1 $2 $3 $4 $5
  fi
}

grant_advanced() {
  local database="$1"
  local privilege="$2"
  local user="$3"
  local password="$4"
  local table="$5"

  sql_exec "GRANT ${privilege} ON ${database}.${table} TO '${user}'@'localhost' IDENTIFIED BY '${password}'"
  sql_exec "GRANT ${privilege} ON ${database}.${table} TO '${user}'@'127.0.0.1' IDENTIFIED BY '${password}'"
  if [ "${REMOTE}" = 1 ]; then
    sql_exec "GRANT ${privilege} ON ${database}.${table} TO '${user}'@'%' IDENTIFIED BY '${password}'"
  fi
}

# Выполняет $1 как SQL-запрос
sql_exec() {
  mysql -u$ANSWER_SOFTWLC_MYSQL_USER -p$ANSWER_SOFTWLC_MYSQL_PASSWORD -e "$1" >/dev/null
  return $?
}

# Контроль открытого порта для определённой службы в режиме UDP (нет LISTEN)
# Пример вызова функции: check_port_service_udp "8080" "nginx" "nginx" result_var
# Пример вызова функции: check_port_service_udp "8080" "java" "tomcat" result_var
# $1 - номер порта (строкой);
# $2 - название сервиса (java - для всех java приложений);
# $3 - описание сервиса, например, 'tomcat' или 'Captive Portal' для Java приложения;
# $4 - В результат будет помещена строка: "0" - это OK (порт открыт) или "1" - это ошибка (порта нет)
function check_port_service_udp() {
  local __resultvar="$4"
  local myresult=0
  local port=":$1"
  local service="$2"
  local service_descr="$3"
  local variable=$(netstat -ulpna | awk '{if (($1 ~ /udp/ || $1 ~ /udp6/) && $4 ~ "'"$port"'" && ($6 ~ "'"$service"'" || $6 ~ /docker-proxy/)) print "passed"}')
  if [[ $variable =~ "passed" ]]; then
    echo "${green}Checking udp port '$1' for application '$2' ($service_descr) - passed${reset}"
    myresult="0"
  else
    echo "${red}Checking udp port '$1' for application '$2' ($service_descr) - error${reset}"
    myresult="1"
  fi
  eval ${__resultvar}="'$myresult'"
}

# Функция для финальной проверки всех портов однохостовой инсталляции
# Считает количество фатальных ошибок (не все порты критичные)
# Если количество фатальных ошибок больше нуля, то после отчёта скрипт прервётся и выйдет с "-1"
function main_ports_test() {
  local FATAL=0
  echo ""
  echo "Start TCP/UDP port checking for all services.."

  # == databases ==
  check_port_service_tcp "3306" "mysqld" "Server MySQL" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "27017" "mongo" "MongoDB server" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == EMS ==
  check_port_service_tcp "9310" "java" "EMS-server Applet API " var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_udp "162" "java" "EMS server SNMP API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == tomcat ==
  check_port_service_tcp "8081" "java" "tomcat (NBI, EMS applet)" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == wifi-customer-cab spring-boot ==
  check_port_service_tcp "8083" "java" "WiFi Customer Cab (Jetty)" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == nginx ==
  check_port_service_tcp "8080" "nginx" "Nginx proxy server" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # Captive Portal
  check_port_service_tcp "9000" "java" "Captive Portal, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # Portal Constructor
  check_port_service_tcp "9001" "java" "Portal Constructor, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # ngw
  check_port_service_tcp "8040" "java" "eltex-notification-gw, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # apb
  check_port_service_tcp "8090" "java" "eltex-apb, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # logging
  check_port_service_tcp "9099" "java" "eltex-logging-service, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # mercury
  check_port_service_tcp "6565" "java" "eltex-mercury service, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # doors
  check_port_service_tcp "9097" "java" "eltex-doors service HTTP API, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # johnny
  check_port_service_tcp "9100" "java" "eltex-johnny service HTTP API, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # eltex-disconnect-service
  check_port_service_tcp "9096" "java" "eltex-disconnect-service HTTP API, Portal Group" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == PCRF ==
  check_port_service_tcp "7070" "java" "PCRF monitoring API" var
  check_port_service_tcp "7080" "java" "PCRF RADIUS API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi
  check_port_service_tcp "5701" "java" "PCRF Hazelcast API" var
  check_port_service_udp "1813" "java" "PCRF, RADIUS accounting API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # eltex-bruce
  check_port_service_tcp "8008" "java" "eltex-bruce service HTTPS API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # eltex-jobs
  check_port_service_tcp "9696" "java" "eltex-jobs service HTTPS API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == RADIUS ==
  check_port_service_udp "1812" "eltex-radius" "RADIUS API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # eltex-airtune
  check_port_service_tcp "8082" "eltex-airtune" "eltex-airtune API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi
  check_port_service_tcp "8089" "eltex-airtune" "eltex-airtune API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi
  check_port_service_tcp "8099" "eltex-airtune" "eltex-airtune API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  # == Other ==
  check_port_service_udp "514" "syslog" "Linux syslog server" var
  check_port_service_udp "69" "tftp" "Linux TFTP server" var

  echo ""
  # Финальный контроль фатальных ошибок
  if [ $FATAL != 0 ]; then
    echo "${red}Found $FATAL port errors, script aborted!${reset}"
    exit 2
  else
    echo "${green}> Main core ports are tested successfully!${reset}"
  fi
  echo ""
}

# Функция для контроля портов работы ядра в режиме OTT
function ott_ports_test() {
  local FATAL=0
  echo ""
  echo "Start TCP/UDP port checking for OTT services.."

  check_port_service_tcp "8088" "java" "eltex-ott-mac-checker, OTT" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "8098" "java" "eltex-paul, password generator, OTT" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "8043" "java" "eltex-wifi-sa, service activator HTTPS API, OTT" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  echo ""
  # Финальный контроль фатальных ошибок
  if [ $FATAL != 0 ]; then
    echo "${red}Found $FATAL errors, script aborted!${reset}"
    exit 2
  else
    echo "${green}> OTT ports are tested successfully!${reset}"
  fi
  echo ""
}

# Функция для контроля портов работы ядра в режиме Clickhouse
function clickhouse_ports_test() {
  local FATAL=0
  echo ""
  echo "Start TCP/UDP port checking for Clickhouse database and services.."

  # == Clickhouse ==
  check_port_service_tcp "9003" "clickhouse" "Clickhouse client API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "9009" "clickhouse" "Clickhouse inter serever API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "8123" "clickhouse" "Clickhouse HTTP API" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  check_port_service_tcp "9070" "eltex-bonnie" "Bonnie GRPC API, Clickhouse" var
  if [[ "$var" != "0" ]]; then
    FATAL=$((FATAL + 1))
  fi

  echo ""
  # Финальный контроль фатальных ошибок
  if [ $FATAL != 0 ]; then
    echo "${red}Found $FATAL errors, script aborted!${reset}"
    exit 2
  else
    echo "${green}> Clickhouse ports are tested successfully!${reset}"
  fi
  echo ""
}

# Валидация IP адреса через regex
function valid_ipv4() {
  local ip=$1
  local stat=1

  if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
    OIFS=$IFS
    IFS='.'
    ip=($ip)
    IFS=$OIFS
    [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]

    stat=$?
  fi
  return $stat
}

# Заполнить массив EXTERNAL_IPS внешними IP сервера
# Для IPv4 — весь ifconfig, кроме 127.0.0.1
function find_external_ipv4_ips() {
  # Debian & U20 не содержат утилиту ifconfig (net-tools) в базовой поставке
  # local ifconfig="$(ifconfig | grep -Po '(?<=inet\s)[^\s]*')"
  local ipv4_addresses=$(ip -4 addr | grep inet | awk -F '[ \t]+|/' '{print $3}' | grep -v ^127.0.0.1)
  for address in ${ipv4_addresses}; do
    # Контроль, что адрес получился валидным. Если нет, то пробуем дополнительно очистить
    if ! valid_ipv4 $address; then
      address="$(echo $address | grep -oE '((1?[0-9][0-9]?|2[0-4][0-9]|25[0-5])\.){3}(1?[0-9][0-9]?|2[0-4][0-9]|25[0-5])')"
    fi
    # Если после очистки всё ОК, то включаем строку в выходной массив, за исключением localhost (127.0.0.1)
    if (valid_ipv4 $address) && [[ ! (${address} == "127.0.0.1") ]]; then
      EXTERNAL_IPS+=($address)
    fi
  done
}

# Заполнить массив EXTERNAL_IPS внешними IP сервера
function find_external_ips() {
  EXTERNAL_IPS=()
  find_external_ipv4_ips
  # TODO: поддержать (протестировать) IPv6; смотри историю коммитов для #206953
}

# Запросить у пользователя внешний IP из EXTERNAL_IPS.
# В $1 функция возвращает значение IP (не индекс)
function select_ip() {
  local __resultvar=$1
  ans=""
  while true; do
    read ans
    if [[ (${ans} -gt 0) && (${ans} -lt ${#EXTERNAL_IPS[@]}+1) ]]; then
      break
    else
      echo "Wrong IP number. Try again"
    fi
  done
  eval ${__resultvar}="${EXTERNAL_IPS[ans - 1]}"
}

# Узнать внешние IP сервера и, если их несколько, предложить пользователю выбор.
# В $1 функция возвращает окончательное значение внешнего IP
function detect_external_ip() {
  local __resultvar=$1
  case ${#EXTERNAL_IPS[@]} in
  0)
    echo "No external IP addresses have been detected"
    host="127.0.0.1"
    ;;
  1)
    echo "External IP address has been detected"
    host=${EXTERNAL_IPS[0]}
    ;;
  *)
    echo "Several external IPs have been detected:"
    for index in "${!EXTERNAL_IPS[@]}"; do
      echo "$(($index + 1)). ${EXTERNAL_IPS[index]}"
    done
    echo "Which one will be used to open the portal constructor and the customer cabinet in a browser?"
    select_ip host
    ;;
  esac
  eval ${__resultvar}="${host}"
}

# Узнать внутренние IP сервера и, если их несколько, предложить пользователю выбор.
# В $1 функция возвращает окончательное значение внутреннего IP
function detect_internal_ip() {
  local __resultvar=$1
  case ${#EXTERNAL_IPS[@]} in
  0)
    echo "No internal IP addresses have been detected"
    host="127.0.0.1"
    ;;
  1)
    echo "Internal IP address has been detected"
    host=${EXTERNAL_IPS[0]}
    ;;
  *)
    echo "Several internal IPs have been detected:"
    for index in "${!EXTERNAL_IPS[@]}"; do
      echo "$(($index + 1)). ${EXTERNAL_IPS[index]}"
    done
    echo "Which one will be used as the management IP (for FTP, TFTP, HTTP and other protocols)?"
    select_ip host
    ;;
  esac
  eval ${__resultvar}="${host}"
}


# Делает валидацию аргумента и прерывает выполнение скрипта.
# Если адрес не валиден или '127.0.0.1', или его нет среди адресов интерфейсов сервера,
# то скрипт прерывается с ошибкой
function validate_server_ip_from_args() {
  # Формальная проверка на правильность адреса
  if ! valid_ipv4 $1; then
    echo "${red}Server IP addr argument is invalid: '$1', script aborted${reset}"
    exit 1
  fi

  # Проверка, что это не localhost (так как этот адрес приведёт пользователя браузера на собственный ПК)
  if [[ $1 == "127.0.0.1" || $1 == "::1" ]]; then
    echo "${red}Cannot assign localhost IP '$1' to server IP address; script aborted${reset}"
    exit 1
  fi

  local found_var=0
  # Получаем список адресов, что есть на данном сервере
  find_external_ips

  # Проводим контроль, что юзер передал один из реальных адресов сервера
  for index in "${!EXTERNAL_IPS[@]}"; do
    echo "$(( index + 1 )). ${EXTERNAL_IPS[index]}"
    if [[ $1 == ${EXTERNAL_IPS[index]} ]]; then
      found_var=1
      break
    fi
  done

  if (( found_var == 0 )); then
    echo "${red}Address not found on interfaces : '$1', script aborted${reset}"
    exit 1
  fi
}

# Функция включает конфигурацию clickhouse (accounting, bonnie) в зависимых службах
# По умолчанию конфигурация стартует с аккаунтингом в MySQL.
function enable_clickhouse_config() {
  sed -i 's/\"use_clickhouse\": false,/\"use_clickhouse\": true,/g' /etc/eltex-pcrf/eltex-pcrf.json
  sed -i 's/\"use_mysql\": true,/\"use_mysql\": false,/g' /etc/eltex-pcrf/eltex-pcrf.json
  sed -i 's/<entry key=\"data.service.type\">NBI<\/entry>/<entry key=\"data.service.type\">bonnie<\/entry>/g' /etc/eltex-wifi-cab/system.xml
}

# Метод для проверки валидности имени дистрибутива.
# Скрипт не будет устанавливать софт на неизвестный дистрибутив Linux
function check_codename_and_platform() {
  # проверить разрядность системы и отказаться работать, если не x64
  local DISTRIB_PLATFORM=$(/bin/uname -m)
  echo "Platform : $DISTRIB_PLATFORM"

  if [[ ${DISTRIB_PLATFORM} != "x86_64" ]]; then
    echo "${red}Platform is not 'x86_64', script aborted!${reset}"
    exit 1
  fi

  # Проверить, что наименование дистрибутора Linux находится в списке известных
  local FOUND=0
  for distr in ${LINUX_DISTRIBUTORS}; do
    if [[ $1 == "$distr" ]]; then
      FOUND=1
      break
    fi
  done

  if [[ $FOUND == 0 ]]; then
    echo "${red}Unsupported Linux Distributor '$1', script aborted.${reset}"
    exit 1
  fi

  # Проверить, что кодовое имя дистрибутива находится в списке известных
  FOUND=0
  for distr in ${LINUX_CODENAMES}; do
    if [[ $2 == "$distr" ]]; then
      FOUND=1
      break
    fi
  done

  if [[ $FOUND == 0 ]]; then
    echo "${red}Unsupported Linux Codename '$2', script aborted.${reset}"
    exit 1
  fi
}

function add_mysql_gpg_key() {
  # Временно отключаем прерывание установок при ошибке, чтобы обработать ее ниже
  set +e
  local get_key_exit_code=0
  # Если с hkp://keyserver.ubuntu.com:80 не получилось достать ключ - пробуем получить его другим путём
  for ((i = 1; i <= 3; i++))
  do
    echo "Trying to receive gpg key from hkp://keyserver.ubuntu.com:80."
    apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv-keys 467B942D3A79BD29
    get_key_exit_code=$?
    if [ $get_key_exit_code -ne 0 ]; then
      echo "Error receiving gpg key from hkp://keyserver.ubuntu.com:80."
      echo "Trying to get gpg key from repo.mysql.com"
      wget -O - https://repo.mysql.com/RPM-GPG-KEY-mysql-2022 | apt-key add -
      get_key_exit_code=$?
    fi

    if [ $get_key_exit_code -eq 0 ]; then
      break
    fi
    echo "Error receiving gpg key from repo.mysql.com"

  done
  set -e

  if [ $get_key_exit_code -ne 0 ]; then
    echo "Error receiving GPG key"
    exit 1
  fi
}

# Инсталлирует MySQL в зависимости от версии операционной системы
function install_mysql_community() {

  # Добавить новый доверенный ключ на репозиторий
  add_mysql_gpg_key

  export DEBIAN_FRONTEND=noninteractive

  # mysql root password settings (default: login=root, password=root)
  debconf-set-selections <<<"mysql-server mysql-server/root_password password ${ANSWER_SOFTWLC_MYSQL_PASSWORD}"
  debconf-set-selections <<<"mysql-server mysql-server/root_password_again password ${ANSWER_SOFTWLC_MYSQL_PASSWORD}"

  if [[ "$ORIGINAL_DISTRIB_CODENAME" == "orel" || "$ORIGINAL_DISTRIB_CODENAME" == "stretch" ]]; then
    apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv-keys 04EE7237B7D453EC
    echo "deb http://archive.debian.org/debian/ stretch main"  >> /etc/apt/sources.list
    echo "deb-src http://archive.debian.org/debian/ stretch main"  >> /etc/apt/sources.list
    apt-get update
    apt-get install -y libmecab2
    apt-get install -y daemon
    apt-get install -y mysql-common=5.7.34-1debian9
    apt-get install -y mysql-client=5.7.34-1debian9
    apt-get install -y mysql-community-server
  else
    # add repository containing mysql-apt-config package
    echo "deb http://repo.mysql.com/apt/ubuntu $DISTRIB_CODENAME mysql-apt-config" >> /etc/apt/sources.list
    apt-get update

    apt-get install -y libmecab2
    apt-get install -y daemon

    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/repo-codename select ${DISTRIB_CODENAME}"
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/repo-distro select ${DISTRIBUTOR_ID}"
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/repo-url string http://repo.mysql.com/apt/"
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/select-preview select "
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/select-product select Ok"
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/select-server select mysql-5.7"
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/select-tools select "
    debconf-set-selections <<<"mysql-apt-config mysql-apt-config/unsupported-platform select abort"

    # download mysql-apt-config package
    rm -rf /tmp/softwlc-mysql/
    mkdir /tmp/softwlc-mysql/
    chown -R _apt:root /tmp/softwlc-mysql/
    local current_dir="$(pwd)"
    cd /tmp/softwlc-mysql/
    apt-get download mysql-apt-config

    # install mysql-apt-config
    dpkg -i $(ls . | grep mysql-apt-config)

    # delete mysql-apt-config package
    cd "${current_dir}"
    rm -rf /tmp/softwlc-mysql/

    apt-get update
    apt-get install -y mysql-community-server
  fi

  # прописать принудительно выключение ssl сервиса mysql для всех целевых ОС
  MYSQL_CFG_FILE=/etc/mysql/mysql.conf.d/mysqld.cnf
  if [[ ! -f "$MYSQL_CFG_FILE" ]]; then
    echo "${red}File not exists '$MYSQL_CFG_FILE'${reset}"
  else
    # Если файл есть, то проверить что в конфиге ssl ещё не выключен
    if [[ ! `egrep "^[^#;]" $MYSQL_CFG_FILE | egrep "ssl=0"` ]]; then
      echo -en '\nssl=0\n' >> $MYSQL_CFG_FILE
      echo "Modified file '$MYSQL_CFG_FILE', restarting service mysql"
      service mysql restart
    fi
  fi
}

# Инсталляция утилиты curl в зависимости от платформы
function install_lib_curl() {
  case "$DISTRIB_CODENAME" in
  "xenial"|"stretch"|"orel")
    LIBCURL_PACKET_NAME="libcurl3"
    ;;
  "bionic"|"buster")
    LIBCURL_PACKET_NAME="libcurl4"
    ;;
  *)
    echo "Unknown Linux codename for libcurl. Script aborted!"
    exit 1
    ;;
  esac
  install $LIBCURL_PACKET_NAME
}

# Инсталляция и настройка пакета libssl. Доступная версия зависит от ОС.
function install_libssl() {
  case "$DISTRIB_CODENAME" in
  "xenial"|"bionic"|"orel")
    LIBSSL_PACKET_NAME="libssl1.0.0"
    ;;
  "buster"|"focal")
    LIBSSL_PACKET_NAME="libssl1.1"
    ;;
  "stretch")
    LIBSSL_PACKET_NAME="libssl1.0.2"
    ;;
  *)
    echo "Unknown Linux codename for libssl. Script aborted!"
    exit 1
    ;;
  esac

  # установить
  install ${LIBSSL_PACKET_NAME}
  }


# Инсталляция и настройка пакета tomcat. Доступная версия зависит от ОС.
function install_tomcat() {
  case "$DISTRIB_CODENAME" in
  "xenial")
    TOMCAT_PACKET_NAME="tomcat7"
    ;;
  "bionic"|"stretch"|"orel")
    TOMCAT_PACKET_NAME="tomcat8"
    ;;
  "buster"|"focal")
    TOMCAT_PACKET_NAME="tomcat9"
    ;;
  *)
    echo "Unknown Linux codename for tomcat. Script aborted!"
    exit 1
    ;;
  esac

  # установить Tomcat
  install ${TOMCAT_PACKET_NAME}

  # Установить максимальный размер heap для Tomcat
  sed -i "s/-Xmx[0-9]\+[a-zA-Z]/-Xmx${ANSWER_TOMCAT_MAX_HEAP}m/" /etc/default/${TOMCAT_PACKET_NAME}

  # Добавить джава опцию, изменяющую рандом файл на urandom
  if ! grep "^[^#].*-Djava\.security\.egd=file:/dev/\./urandom\b" /etc/default/${TOMCAT_PACKET_NAME} &>/dev/null; then
    echo "JAVA_OPTS=\"\${JAVA_OPTS} -Djava.security.egd=file:/dev/./urandom\"" >>/etc/default/${TOMCAT_PACKET_NAME}
  fi

  # Принудительное использование openjdk в Tomcat
  echo "JAVA_HOME=/usr/lib/jvm/java-8-openjdk-amd64" >>/etc/default/${TOMCAT_PACKET_NAME}
}

# Метод для полной инсталляции rsyslog-mysql, с заменой конфигов, модификацией баз,
# выдачей GRANT и всем остальным.
function full_installation_rsyslog_mysql() {
  install rsyslog-mysql

  # Пересоздать схему 'Syslog' в базе данных (в случае необходимости)
  res="0"
  rsyslog_check_extended_database res
  if [[ "$res" == "1" ]]; then
    echo "${red}Invalid database 'Syslog', need update ${reset}"
    rsyslog_create_extended_database
    echo "${green}Database 'Syslog' updated ${reset}"
  else
    echo "${green}Table 'SystemEvents' is valid (contains parts)${reset}"
  fi

  # После пересоздания БД лучше ещё раз её проверить, чтобы убедиться, что всё в базе согласно ожиданий
  rsyslog_check_extended_database res
  if [[ "$res" == "1" ]]; then
    echo "${red}Invalid database 'Syslog', update failed, script aborted!${reset}"
    exit 1
  fi

  # Выдать права (гранты) на вновь созданную схему для пользователя в СУБД, с которым работает продукт EMS
  grant "Syslog" ALL "$EMS_USER" "$EMS_PASSWORD"

  # После установки rsyslog-mysql нужно провести его кастомную конфигурацию.
  # Включить приём данных из сети по UDP и TCP
  # Настроить, чтобы сетевой трафик сохранялся в mysql и не сохранялся в локальные файлы
  rsyslog_uncomment_network_mod
  rsyslog_mysql_replace_config
  # Рестарт службы для применения всех новых конфигов
  restart rsyslog
}

function configure_rsyslog() {
  # Устанавливаем права на файлы eltex-radius, radius.log
  user_name=""
  case "$DISTRIBUTOR_ID" in
  "ubuntu")
     user_name="syslog"
    ;;
  "debian")
     user_name="root"
    ;;
  *)
    echo "Unknown Linux distribution. Script aborted!"
    exit 1
    ;;
  esac

  chown $user_name:adm /var/log/eltex-radius
  chown $user_name:adm /var/log/eltex-radius/radius.log

  # сохраняем путь до файла rsyslog-rotate в переменную
  ROTATE_FILE=/usr/lib/rsyslog/rsyslog-rotate
  # каким должен быть файл rsyslog-rotate
  rotate_script_correct="#!/bin/sh

if [ -d /run/systemd/system ]; then
   systemctl kill -s HUP rsyslog.service
else
   invoke-rc.d rsyslog rotate > /dev/null
fi"

  # Проверяем существование файла, если такого файла нет - создаем с нужным содержимым
  # Если он существует - не трогаем, чтобы не затереть пользовательские настройки
  if [[ -f "$ROTATE_FILE" ]]; then
    echo File $ROTATE_FILE already exists
  else
    if [[ ${DISTRIB_CODENAME} == "stretch" ]]; then
        mkdir /usr/lib/rsyslog
        sed -i 's|invoke-rc.d rsyslog rotate > /dev/null|/usr/lib/rsyslog/rsyslog-rotate|' /etc/logrotate.d/rsyslog
    fi
    echo "$rotate_script_correct" > $ROTATE_FILE
    echo File $ROTATE_FILE has been successfully created
  fi

  chmod 755 $ROTATE_FILE

  FILE=/etc/logrotate.d/eltex-radius
  # раскомментировать строки и поставить 8 пробелов для выравнивания исходного форматирования
  sed -i '/sharedscripts/s/^\s*#/        /g' $FILE
  sed -i '/postrotate/s/^\s*#/        /g' $FILE
  sed -i '/\/usr\/lib\/rsyslog\/rsyslog-rotate/s/^\s*#/        /g' $FILE
  sed -i '/endscript/s/^\s*#/        /g' $FILE

  restart rsyslog
}

# Копирование файлов лицензии eltex-ems для eltex-airtune
function copy_ems_license_to_airtune {
  LICENSE_FILES_ARRAY=()
  for f in $AIRTUNE_LICENSE_FILES; do
    if [[ -f "$f" ]]; then
      LICENSE_FILES_ARRAY+=("$f")
    fi
  done

  if [ ${#LICENSE_FILES_ARRAY[@]} -eq 0 ]; then
    echo "No old license files found"
  else
    echo "Found ${#LICENSE_FILES_ARRAY[@]} old license files. Renaming to '*.old':"
    for f in ${LICENSE_FILES_ARRAY[@]}; do
      if [ -f "$f.old" ]; then
        echo "$f.old file already exists. Rename or delete this file. Skipped."
      else
        mv --verbose "$f" "$f.old"
      fi
    done
  fi
  echo

  echo "Copying eltex-ems license files for eltex-airtune from '$EMS_LICENSE_PATH' folder:"
  for f in $EMS_LICENSE_FILES; do
    if [ -f "$AIRTUNE_LICENSE_PATH/${f##*/}" ]; then
      echo "File ${f##*/} already exists in airtune license folder. Skip copying."
    else
      cp -n --verbose "$f" "$AIRTUNE_LICENSE_PATH/"
    fi
  done
}

# Самое первое - это запрет работы не от root
if [[ $(id -u) -ne 0 ]]; then
  echo "${red}This script can only be run as root${reset}"
  exit 1
fi

# Читаем и применяем аргументы командной строки
if [[ -n "$1" ]]; then
  for i in "$@"; do
    # Определить repo: public | internal
    if [[ ${i} == "--public" ]]; then
      ELTEX_REPO=${ELTEX_PUBLIC_REPO}
      update_repo_related_vars
    fi

    if [[ ${i} == "--private" ]]; then
      ELTEX_REPO=${ELTEX_PRIVATE_REPO}
      update_repo_related_vars
    fi

    ## Set the variable INSTALL_CLICKHOUSE
    if [[ ${i} == "--clickhouse" ]]; then
      INSTALL_CLICKHOUSE=1
    fi

    ## Set the variable INSTALL_DHCP
    if [[ ${i} == "--dhcp" ]]; then
      INSTALL_DHCP=1
    fi

    ## Set the variable OTT
    if [[ ${i} == "--ott" ]]; then
      WITH_OTT=1
    fi

    ## Server IP settings (external)
    if [[ ${i} =~ "serverip=" ]] || [[ ${i} =~ "SERVERIP=" ]]; then
      external_ip=$(echo ${i} | cut -d "=" -f 2)
      # Валидация с реальным списком адресов на интерфейсах. Если ошибка, то выход из скрипта.
      validate_server_ip_from_args $external_ip
    fi

    ## Server IP settings (internal=management)
    if [[ ${i} =~ "emsip=" ]] || [[ ${i} =~ "EMSIP=" ]]; then
      ems_ip=$(echo ${i} | cut -d "=" -f 2)
      # Валидация с реальным списком адресов на интерфейсах. Если ошибка, то выход из скрипта.
      validate_server_ip_from_args $ems_ip
    fi

    ## Ports test mode
    if [[ ${i} == "--test-ports" ]]; then
      TEST_PORTS_MODE=1
    fi

    if [[ ${i} == "--force-new-conffiles" ]]; then
      echo "Choosed new conffiles strategy for dpkg. Proceed"
      NEW_CONF_FILES=1
    fi

    if [[ ${i} == "--force-old-conffiles" ]]; then
      echo "Choosed old conffiles strategy for dpkg. Proceed"
      OLD_CONF_FILES=1
    fi

    if [[ ${NEW_CONF_FILES} == 1 && ${OLD_CONF_FILES} == 1 ]]; then
      echo "Can't use both dpkg update strategies. Please choose only one"
      exit 1
    fi

  done
fi

update
# установить (обновить) программу работы с репозиторием add-apt-repository (т.к. в некоторых системах она не присутствует)
install software-properties-common
install lsb-release wget
# Узнать наименование дистрибутора (debian/ubuntu/astra-linux) и записать lowercase
tmp_str=$(lsb_release -is)
DISTRIBUTOR_ID="${tmp_str,,}"

# узнать кодовое имя дистрибутива и записать lowercase
tmp_str=$(lsb_release -cs)
DISTRIB_CODENAME="${tmp_str,,}"
ORIGINAL_DISTRIB_CODENAME="${tmp_str,,}"

check_codename_and_platform ${DISTRIBUTOR_ID} ${DISTRIB_CODENAME}

echo "${green}OS distributer ID: $DISTRIBUTOR_ID${reset}"
echo "${green}OS distrib code name: $DISTRIB_CODENAME${reset}"
echo "${green}Repository: $ELTEX_REPO ${reset}"
echo "${green}Java vendor: $JAVA_VENDOR${reset}"

# Сохраним наименование дистрибуции для пакета eltex-radius, eltex-radius-db, он единственный дистрибутится в персональном репе
RADIUS_DISTRIBUTION="$RADIUS_VERSION-$DISTRIB_CODENAME"

# Astralinux(ce) 'orel' - это форк Debuan 9 'stretch', поэтому чтобы работали все репозитории (Элтекс и весь opensource),
# мы подменяем в скрипте $CODENAME 'orel' на 'stretch' $DISTRIBUTOR_ID 'astralinuxce' на 'debian' и работаем далее.
if [[ ${DISTRIBUTOR_ID} == "astralinuxce" ]]; then
  DISTRIBUTOR_ID="debian"
  echo "${green}Replaced OS distributer ID: $DISTRIBUTOR_ID${reset}"
fi
if [[ ${DISTRIB_CODENAME} == "orel" ]]; then
  DISTRIB_CODENAME="stretch"
  echo "${green}Replaced OS distrib code name: $DISTRIB_CODENAME${reset}"
fi

# Режим "только контроль портов". Если он включен, то контролируем порты и завершаем программу
# Скрипт завершится со статусом 0, если все проверки удачные
# Скрипт завершится со статусом 2, если есть неоткрытые порты в основной (main) или доп.секциях (ott, clickhouse)
if [[ ${TEST_PORTS_MODE} -eq 1 ]]; then
  echo "Test ports mode"
  # Core ports
  main_ports_test
  # OTT ports
  if [[ ${WITH_OTT} == 1 ]]; then
    ott_ports_test
  fi
  # clickhouse ports
  if [[ ${INSTALL_CLICKHOUSE} == 1 ]]; then
    clickhouse_ports_test
  fi
  exit 0
fi

# найти все адреса на сервере (кроме локальных)
find_external_ips

# Если внешний IP адрес сервера (для настроек ЛК и КП) не задан аргументами скрипта,
# то его нужно спросить у пользователя и не двигаться дальше, пока не получим значение
if [ -z "$external_ip" ]; then
  echo "Setting portal constructor and customer cabinet link addresses.."
  detect_external_ip external_ip
  echo "${green}"Server external IP is $external_ip"${reset}"
  echo
else
  echo "${green}"Server is defined from args: $external_ip"${reset}"
  echo
fi

# Если внутренний IP адрес сервера (для настроек EMS, FTP, TFTP и т.д.) не задан аргументами скрипта,
# то его нужно спросить у пользователя и не двигаться дальше, пока не получим значение
if [ -z "$ems_ip" ]; then
  echo "Setting EMS addresses.."
  detect_internal_ip ems_ip
  echo "${green}"Server internal IP is $ems_ip"${reset}"
  echo
else
  echo "${green}"Server is defined from args: $ems_ip"${reset}"
  echo
fi

check_memory

# выполнение 'clean_eltex_repo' ДО любых действий с утилитой apt, т.к. переключение repo
# может приводить к недоступности repo ('private' vs 'public') и скрипт прервётся на apt update
clean_eltex_repo

# получить конфигурацию nginx соотв. версии
wget ${ELTEX_REPO}/nginx/conf/${NGINX_CONFIG_FILE} -O ${NGINX_CONFIG_FILE} || true

check_nginx_config

# Путь репозитория НЕ переопределяется в зависимости от имени дистрибутива Ubuntu.
# Все JAVA проекты (ом независымые) будут в репозитории по имени xenial
SOFTWLC_DISTRIBUTION="$SOFTWLC_VERSION-xenial"

if [[ ${DISTRIB_CODENAME} == "xenial" ]]; then
  # обновить репозиторий (перед инсталляцией gnupg-curl, иначе он недоступен!)
  apt-get update || true

  # установка пакета 'gnupg-curl', без которого в некоторых системах невозможно выкачать и проверить 'keyserver'
  install gnupg-curl
fi

# добавить основной репозиторий Eltex в список источников apt
echo "deb [arch=amd64] $ELTEX_REPO $SOFTWLC_DISTRIBUTION main" >>/etc/apt/sources.list.d/eltex.list
# добавить дополнительный репозиторий для пакета eltex-radius в список источников apt
echo "deb [arch=amd64] $ELTEX_REPO $RADIUS_DISTRIBUTION main" >>/etc/apt/sources.list.d/eltex.list

wget -O - ${REPO_GPG_KEY_ADDR} | apt-key add -

# Ставим всегда самый новый nginx с родного репо проекта (иначе несовместимость по конфигам старых версий)
wget https://nginx.org/keys/nginx_signing.key
apt-key add nginx_signing.key

echo "deb http://nginx.org/packages/$DISTRIBUTOR_ID/ $DISTRIB_CODENAME nginx" >/etc/apt/sources.list.d/nginx.list

# Инсталляция системных пакетов
# обновить репозиторий
update

# Заранее задать ответы на вопросы инсталлятора для rsyslog
set_rsyslog_mysql_silent_mode

# провести инсталляцию пакета openjdk
install openjdk-8-jdk
# прописать в системе использование только что установленного пакета
update-java-alternatives -s java-1.8.0-openjdk-amd64

# Обязательное создание линка ДО инсталляции Tomcat
# иначе процесс для Tomcat вывалится с ошибкой 'Not found JAVA_HOME' и вся установка прервётся.
# Обнаружено в Ubuntu16 (xenial), в старой Ubuntu14 (trusty) всё проходило нормально.
if [[ ! -e /usr/lib/jvm/default-java ]]; then
  ln -s /usr/lib/jvm/java-8-openjdk-amd64 /usr/lib/jvm/default-java
fi

# Инсталляция tomcat только после JVM
install_tomcat

# установить прочие пакеты, которые прописаны в зависимостях пакетов
install expect psmisc ntp tftp-hpa tftpd-hpa snmpd snmp rsyslog curl libpcap0.8 fping vsftpd lockfile-progs \
libstdc++6 zlib1g

# инсталляция библиотеки зависит от версии ОС (либа нужна для eltex-airtune)
install_libssl

# Пакеты для работы видео на портале и в конструкторе. Метапакет ffmpeg.
install ffmpeg

# некоторые новые дистрибутивы (desktop) могут не содержать net-tools:netstat
install net-tools

# Установка библиотеки libcurl, которая очень нужна некоторым сервисам, например eltex-radius, причём индекс
# зависит от операционной системы
install_lib_curl

# установить MongoDB
install mongodb-org
systemctl enable mongod.service
service mongod restart

# Установить mysql из специального репозитория, где нужная версия доступна для всех ОС.
# Устанавливать обязательно после установки libpcap0.8, libssl, openssl
install_mysql_community

# переписываем лимиты для сервиса mysql, иначе rsyslog-mysql с пагинацией не работает на различных ОС
replace_open_files_for_mysql

# rsyslog-mysql устанавливается только после окончания установки mysql-server
# иначе попытка его конфигурирования провалится
full_installation_rsyslog_mysql

# По отдельному флагу инсталлируем DHCP (настройка подсетей и запуск на совести сисадмина!)
if [[ ${INSTALL_DHCP} == 1 ]]; then
  install isc-dhcp-server
fi

## Install ClickHouse
if [[ ${INSTALL_CLICKHOUSE} -eq 1 ]]; then

  echo "Install ClickHouse"
  echo "deb http://repo.yandex.ru/clickhouse/deb/stable/ main/" >/etc/apt/sources.list.d/clickhouse.list
  install dirmngr                                                       # optional
  apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv E0C56BD4 # optional
  update
  install clickhouse-client clickhouse-server

  ## Install bonnie-db with ClickHouse configuration files
  echo "Install eltex-bonnie-db"
  install eltex-bonnie-db

  ## Start clickhouse-server
  echo "Start clickhouse-server.."
  restart clickhouse-server

  ## Health check
  getresult=$(systemctl is-active clickhouse-server.service)

  if [[ ${getresult} == "active" ]]; then
    echo "${green}. done${reset}"
  elif [[ ${getresult} == "inactive" ]]; then
    echo "${red}. fail. Look the logs in /var/log/clickhouse-server${reset}"
    exit 1
  fi
fi

PACKAGES="eltex-oui-list
        eltex-axis
        eltex-ems-db
        eltex-radius-db
        eltex-auth-service-db
        eltex-ems
        eltex-radius
        eltex-radius-nbi
        eltex-doors
        eltex-ngw
        eltex-apb
        eltex-pcrf
        eltex-logging-service
        eltex-mercury
        eltex-portal
        eltex-portal-constructor
        eltex-wifi-cab
        eltex-disconnect-service
        eltex-johnny
        eltex-bruce
        eltex-jobs
        eltex-airtune"

if [[ ${WITH_OTT} == 1 ]]; then
  PACKAGES="$PACKAGES
        eltex-wifi-sa
        eltex-ott-paul
        eltex-ott-mac-checker"
fi

if [[ ${INSTALL_CLICKHOUSE} == 1 ]]; then
  PACKAGES="$PACKAGES
        eltex-bonnie"
fi

# Последовательная установка пакетов
for package in ${PACKAGES}; do
  echo
  echo "*"
  echo "* Installing $package ..."
  echo "*"
  echo

  install ${package}

  case "$package" in
  eltex-radius-nbi)
    # Установить серверный сертификат в eltex-radius
    /var/lib/eltex-radius-nbi/setup_er_eap.sh
    ;;
  esac
done

# Настройка rsyslog
configure_rsyslog

# Модификации tomcat
TOMCAT_BASE=
TOMCAT_SERVICE=
source "/var/lib/eltex-ems/tomcat-consts"

stop ${TOMCAT_SERVICE}

# Добавить RemoteIpValve в server.xml

VALVE="        <!-- #96184 forward remote ip for nginx -->\n\
        <Valve className=\\\"org.apache.catalina.valves.RemoteIpValve\\\"\n\
           remoteIpHeader=\\\"X-Forwarded-For\\\"\n\
           internalProxies=\\\"127\\\\.0\\\\.0\\\\.1\\\"\n\
           requestAttributesEnabled=\\\"true\\\"/>\n"

AWK="BEGIN {
print \"Input file:\", ARGV[1];
OUTFILE = ARGV[1] \"_\";
print \"Temporary output file:\", OUTFILE;
ALREADY=0;
}
/org\.apache\.catalina\.valves\.RemoteIpValve/ {
ALREADY=1;
exit;
}
/<\/Host>/ {
print \"$VALVE\" > OUTFILE;
}
{
print > OUTFILE;
}
END {
if (ALREADY==0) {
print \"Replace original file '\" ARGV[1] \"' content\";
system(\"cat \" OUTFILE \" > \" ARGV[1]);
}
print \"Remove temporary file '\" OUTFILE \"'\";
system(\"rm \" OUTFILE);
}
"

awk "$AWK" ${TOMCAT_BASE}/conf/server.xml

# Изменить порт tomcat на 8081

PORT8081="    <Connector port=\\\"8081\\\" protocol=\\\"HTTP/1.1\\\""

AWK8081="BEGIN {
print \"Input file:\", ARGV[1];
OUTFILE = ARGV[1] \"_\";
print \"Temporary output file:\", OUTFILE;
ALREADY=0;
}
/Connector port=\"8081\"/ {
ALREADY=1;
exit;
}
/Connector port=\"8080\"/ {
print \"$PORT8081\" > OUTFILE;
next;
}
{
print > OUTFILE;
}
END {
if (ALREADY==0) {
print \"Replace original file '\" ARGV[1] \"' content\";
system(\"cat \" OUTFILE \" > \" ARGV[1]);
}
print \"Remove temporary file '\" OUTFILE \"'\";
system(\"rm \" OUTFILE);
}
"

awk "$AWK8081" ${TOMCAT_BASE}/conf/server.xml

# Установка Nginx

install nginx
cp ${NGINX_CONFIG_FILE} /etc/nginx/conf.d/softwlc.conf

# Записать wifi-cab secret в параметры EMS, чтобы API работало корректно:
#
#$ cat /etc/eltex-wifi-cab/local_secret into eltex_ems.PARAMS
#+----------------+----------------+----------------------------------+
#| param1         | param2         | value                            |
#+----------------+----------------+----------------------------------+
#| wirelessCommon | wificab.secret | 12b3dc9c63c8c75e307855398ac4fbaf |
#+----------------+----------------+----------------------------------+

CAB_KEY="unknown"
CAB_KEY_FILE="/etc/eltex-wifi-cab/local_secret"

while read -r line; do
  CAB_KEY="$line"
  #echo "secret : $key"
done <"$CAB_KEY_FILE"

mysql -ujavauser -pjavapassword -e "INSERT INTO eltex_ems.PARAMS (id, param1, param2, value) \
 VALUES(1, 'wirelessCommon', 'wificab.secret', '$CAB_KEY') \
 ON DUPLICATE KEY UPDATE param1='wirelessCommon', param2='wificab.secret', value='$CAB_KEY';"

# Обновление блока конфигурации EMS сервера в БД internal IP
mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('airtune', 'airtune.api.host', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('ftpserver', 'ftp.addr', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('gPon', 'gpon.ont.tftp.host', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('gePon', 'tftp_host_no_vlan', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('system', 'ip.adress', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('tftpserver', 'tftp.addr', '$ems_ip');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('system', 'tomcat.urlembeded', 'http://$ems_ip:8080');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

echo "Database eltex_ems for internal EMS IP '$ems_ip' updated"

# Задать в конфигурации сервера EMS внешний Tomcat URL
mysql_command="REPLACE INTO eltex_ems.PARAMS (param1, param2, value) VALUES ('system', 'tomcat.url', 'http://$external_ip:8080');"
mysql -ujavauser -pjavapassword -e "$mysql_command"

echo "Database eltex_ems for external EMS IP '$external_ip' updated"

mongo_command="db.config.update({}, {\$set: {\"portal.url\": \"http://$external_ip:9001/epadmin/\"}})"
mongo wifi-customer-cab --eval "$mongo_command"
echo "Portal constructor link in customer cabinet created"
echo

echo "Try to create customer cabinet link in portal constructor (100 seconds for retry).."
CHECK_COUNT=20
PROP_EXISTS="1"
for i in $(seq 1 ${CHECK_COUNT}); do
  properties_table_exists=$(mysql -ujavauser -pjavapassword -e "SHOW TABLES FROM ELTEX_PORTAL LIKE 'properties';")
  if [[ -z "$properties_table_exists" ]]; then
    echo "'ELTEX_PORTAL.properties' table does not exist yet. Waiting for 5 sec and trying again. Attempt ${i}/${CHECK_COUNT}"
    sleep 5
  else
    mysql_command="UPDATE ELTEX_PORTAL.properties SET value = \"$external_ip\" WHERE group_id = 7 AND name = \"host\";"
    mysql -ujavauser -pjavapassword -e "$mysql_command"
    echo "Customer cabinet link in portal constructor created"
    PROP_EXISTS="0"
    break
  fi
done

#Если все циклы проверки прошли, а таблица ELTEX_PORTAL.properties так и не создалась, значит беда, выходим отсюда
if [[ "$PROP_EXISTS" != "0" ]]; then
  echo "${red}Table 'ELTEX_PORTAL.properties' doesn't exist${reset}"
  exit 2
fi

echo "Creating database user and access grants for eltex-ngw"
eltex-ngw create-db-user -u "$ANSWER_SOFTWLC_MYSQL_USER" -p "$ANSWER_SOFTWLC_MYSQL_PASSWORD"

# Скорректировать конфиги служб, если включен режим clickhouse
if [[ ${INSTALL_CLICKHOUSE} == 1 ]]; then
  enable_clickhouse_config
fi

copy_ems_license_to_airtune

echo "Restarting all eltex services.."
# Перезапустить сервисы
restart ${TOMCAT_SERVICE}
restart nginx
restart eltex-radius
restart eltex-doors
restart eltex-pcrf
restart eltex-apb
restart eltex-ngw
# EMS (stop, start)
stop eltex-ems
start eltex-ems
# main other
restart eltex-mercury
restart eltex-logging-service
restart eltex-portal
restart eltex-portal-constructor
restart eltex-bruce
restart eltex-jobs
restart eltex-johnny
restart eltex-wifi-cab
restart eltex-airtune

if [[ ${INSTALL_CLICKHOUSE} -eq 1 ]]; then
  restart eltex-bonnie
fi

if [[ ${WITH_OTT} == 1 ]]; then
  restart eltex-wifi-sa
  restart eltex-ott-paul
  restart eltex-ott-mac-checker
fi

# проверить открытые порты
echo "Waiting 120 seconds.."
echo -ne '##                        (10%)\r'
sleep 12
echo -ne '####                      (20%)\r'
sleep 12
echo -ne '######                    (30%)\r'
sleep 12
echo -ne '########                  (40%)\r'
sleep 12
echo -ne '##########                (50%)\r'
sleep 12
echo -ne '############              (60%)\r'
sleep 12
echo -ne '################          (70%)\r'
sleep 12
echo -ne '##################        (80%)\r'
sleep 12
echo -ne '####################      (90%)\r'
sleep 12
echo -ne '######################    (100%)\r'
echo -ne '\n'

# Проверка на то, что все сервисы успели подняться
CHECK_COUNT=25
echo "Checking services ports ($CHECK_COUNT attempts for check)"
check_all_ports "$CHECK_COUNT" "$WITH_OTT" "$INSTALL_CLICKHOUSE"

# тестирование всех портов комплекса, если один из жизненно важных портов не будет открыт
# будет выход внутри функции: exit 2

# Ждём ещё 10 секунд, т.к. между инициализацией порта 9310 и SNMP-engine проходит долгое время внутри EMS
echo "Waiting 10 seconds.."
sleep 10

main_ports_test

if [[ ${WITH_OTT} == 1 ]]; then
  ott_ports_test
fi

if [[ ${INSTALL_CLICKHOUSE} == 1 ]]; then
  clickhouse_ports_test
fi

echo "Checking EMS internal NBI"
if [[ $(curl "localhost:8080/northbound/getVersion") ]]; then
  echo "${green}Checking EMS.NBI on 'localhost' - passed${reset}"
else
  echo "${red}Checking EMS.NBI on 'localhost' - error${reset}"
  exit 2
fi

echo "Checking mongodb server and database.."
if [[ $(mongo pcrf --eval "printjson(db.getCollectionNames())") ]]; then
  echo "${green}Checking mongodb database PCRF - passed${reset}"
else
  echo "${red}Checking mongodb database PCRF - error${reset}"
  exit 2
fi
if [[ $(mongo wifi-customer-cab --eval "printjson(db.getCollectionNames())") ]]; then
  echo "${green}Checking mongodb database wifi-customer-cab - passed${reset}"
else
  echo "${red}Checking mongodb database wifi-customer-cab - error${reset}"
  exit 2
fi

# Всё
echo ""
echo "Installation of Eltex SoftWLC finished successfully."
echo ""

if [[ ${INSTALL_DHCP} == 1 ]]; then
  echo "${green}DHCP server isc-dhcp-server installed, please configure and start it.${reset}"
fi

echo ""
echo "URLs of SoftWLC components:

Eltex.EMS Server management (internal) IP: $ems_ip
Eltex.EMS Server external IP: $external_ip

Eltex.EMS GUI: http://$external_ip:8080/ems/jws
    login: admin
    password: <empty>

Portal constructor: http://$external_ip:8080/epadmin
    login: $ANSWER_AUTH_SERVICE_ADMIN_USER
    password: $ANSWER_AUTH_SERVICE_ADMIN_PASSWORD

Wi-Fi customer cabinet (B2B): http://$external_ip:8080/wifi-cab
    login: $ANSWER_AUTH_SERVICE_ADMIN_USER
    password: $ANSWER_AUTH_SERVICE_ADMIN_PASSWORD"

exit 0